// Tree Gauge Version
// for Tree generating, Gmsh is used to generate the tree
// path for gauging.

Include "inductive_heat_id.geo";

Dir="inductive_heat/";

ExtGmsh     = ".pos";
ExtGnuplot  = ".dat";

Group {
  CoreE = Region[ {ECORE} ];
  CoreI = Region[ {ICORE} ];
//  Core = Region[ {CoreE, CoreI} ];
  Core = Region[ {CoreE} ];

  Inds      = Region[{COIL}] ;
  SkinInds  = Region[{SKINCOIL}] ;

  Skin_ECore = Region[ {SKINECORE} ];
  Skin_ICore = Region[ {SKINICORE} ];
//  SkinCore = Region[{Skin_ECore, Skin_ICore}];
  SkinCore = Region[{Skin_ECore}];

  Air  = Region[ AIR ];
  AirInf = Region[ AIRINF ];

  // Surfaces for imposing boundary conditions
  Surf_Inf = Region[ SURF_AIROUT ];

  DomainDummy = Region[ 12345 ];
}

Include "materials.data";

Function {
  DefineConstant[
  ];

  vDir[] = ( Vector [-Sin[Atan2[Z[],X[]]#1], 0, Cos[#1]] ); 
  //#1: register of 1 refer to p22 getdp manual

  II = 10000; // ampare
  SurfInd[] = SurfaceArea[]{SURF_ELEC0};
  js0[Inds] = II / SurfInd[] * vDir[] ; // DomainS

  // Use if linear problem
  DefineConstant[
    Freq = { 25.e3, Min 0, Max 100e3, Step 100,
      Name "Input/21Frequency [Hz]", Highlight "AliceBlue"},
    PanDensity = {rhoFe, Min 0, Step 10, Label "Pan density [K/m3]", Name "Material/Pan density"},
    PanSpecificHeat = {cFe, Min 0, Step 10, Label "Pan specific heat [J/kgK]", Name "Material/Pan specific heat"},
    PanThermalConductivity = {kFe, Min 0, Step 10, Label "Pan thermal conductivity [W/m K]",
      Name "Material/Pan thermal conductivity"},
//    sigma_coil = { sigma_cu, Label "Conductivity [S/m]",
//      Name "Input/4Coil Parameters/5Conductivity", Highlight "AliceBlue"},
    PanPermeability = { mu_Fe, Min 100, Max 2000, Step 100,
      Name "Input/42Core relative permeability", Highlight "AliceBlue"},
    PanConductivity = { sigma_Fe, Label "Pan conductivity [S/m]",
      Name "Material/Pan conductivity", Highlight "AliceBlue"},

    PanEmittivity = { eFe, Min 0, Max 1, Label "Pan emittivity",
      Name "Material/Pan emittivity", Highlight "AliceBlue"},

    AmbConvectionCoeff = {10, Min 0, Step 1, Label "Heat transfer coef.[W/m2 K]",
      Name "Ambient Fluid/Heat transfer coef."},

    FinalTime = {10*60, Min 0, Step 60, Name "ThermalAnalysis/Final time"},
    NumSteps = {20, Min 0, Step 10, Name "ThermalAnalysis/Num steps"}
  ];

  Ht[Skin_ECore] = AmbConvectionCoeff;
  Ep[Skin_ECore] = 1.0;
  Cp[CoreE] = PanSpecificHeat;
  K[CoreE] = PanThermalConductivity;
  Tamb[] = 273 + 20;
  //
  time0 = 0;
  time1 = FinalTime;
  dt[] = (time1 - time0)/NumSteps;
  theta = 1.0;
  //
  NL_Eps = 1.0e-8;
  NL_Relax = 1.;
  NL_NbrMax = 1000;
  
}

Group {
  DefineGroup[
    Domain, DomainCC, DomainC,
    DomainS,
    SkinDomainC,
    Surf_Inf, Surf_FixedMVP
  ] ;
}


Function {
  DefineFunction[
    mu, nu, sigma, rho, js
  ] ;

  DefineConstant[
    Freq, T = 1/Freq, // Fundamental period in s
    II, VV
  ] ;

}

Group {
  DomainCC = Region[ {Air, CoreI, AirInf} ];
  DomainC  = Region[ {Core} ];
  SkinDomainC = Region[ {SkinCore} ];

  DomainCwSkin = Region[ {Core,SkinCore} ];
  
  //--------------------------------------------------------------

  DomainS = Region[ {Inds} ];
  DomainCC += Region[ {DomainS} ];

  //--------------------------------------------------------------

  Domain  = Region[ {DomainCC, DomainC} ];

  Surf_FixedMVP = Region[{ Surf_Inf }];

}

Function {
  nu [ Region[{Air, AirInf, Inds}] ]  = 1./mu0 ;
  nu [CoreI]  = 1/(mu_Fe*mu0) ;
  nu [Core] = 1./(PanPermeability*mu0);

//  sigma[Inds] = sigma_coil ;
  sigma[Core] = PanConductivity ;
  rho[Core] = PanDensity;
}


// --------------------------------------------------------------------------

Jacobian {
  { Name Vol ;
    Case { 
           { Region All ;       Jacobian Vol ; }
    }
  }
  { Name Sur ;
    Case { { Region All ; Jacobian Sur ; }
    }
  }
}

Integration {
  { Name II ;
    Case {
      {
	Type Gauss ;
	Case {
	  { GeoElement Triangle    ; NumberOfPoints  4 ; }
	  { GeoElement Quadrangle  ; NumberOfPoints  4 ; }
	  { GeoElement Tetrahedron ; NumberOfPoints  4 ; }
	  { GeoElement Hexahedron  ; NumberOfPoints  6 ; }
	  { GeoElement Prism       ; NumberOfPoints  21 ; }
	  { GeoElement Line        ; NumberOfPoints  4 ; }
	}
      }
    }
  }
}

// --------------------------------------------------------------------------

Group {
  Surf_a_NoGauge = Region [ {Surf_FixedMVP, SkinDomainC} ] ;
}

Constraint {

  { Name GaugeCondition_a ; Type Assign ;
    Case {
      { Region DomainCC ; SubRegion Surf_a_NoGauge ; Value 0. ; }
    }
  }
  // av - formulation
  { Name MVP_3D ;
    Case {
      { Region Surf_Inf ; Type Assign ; Value 0. ; }
    }
  }
  { Name Pot ;
    Case {
      { Region Surf_Inf ; Type Assign ; Value 0. ; }
      { Region SkinDomainC ; Type Assign ; Value 0. ; }
    }
  }
  { Name InitTemp;
    Case {
      { Region DomainC ; Type Init; Value Tamb[] ; }
    }

  }
}

FunctionSpace {

  // Magnetic vector potential a (b = curl a)
  { Name HSpace ; Type Form1 ;
    BasisFunction {// a = a_e * s_e
      { Name se ; NameOfCoef ae ; Function BF_Edge ;
        Support Domain ; Entity EdgesOf[ All, Not SkinDomainC ] ; }
      { Name se2 ; NameOfCoef ae2 ; Function BF_Edge ;
        Support Domain ; Entity EdgesOf[ SkinDomainC ] ; }
    }
    Constraint {
      { NameOfCoef ae  ; EntityType EdgesOf ; NameOfConstraint MVP_3D ; }
      { NameOfCoef ae2 ; EntityType EdgesOf ; NameOfConstraint MVP_3D ; }
      { NameOfCoef ae  ; EntityType EdgesOfTreeIn ; EntitySubType StartingOn ;
	NameOfConstraint GaugeCondition_a ; }
    }
  }

  // Electric scalar potential (3D)
  { Name USpace ; Type Form0 ;
    BasisFunction {
      { Name sr ; NameOfCoef ur ; Function BF_Node ;
        Support DomainC ; Entity NodesOf[ All ] ; }
    }
    Constraint {
      { NameOfCoef ur ; EntityType NodesOf ; NameOfConstraint Pot ; }
    }
  }

  { Name TSpace ; Type Form0;
    BasisFunction {
      { Name sn; NameOfCoef t; Function BF_Node;
	Support DomainCwSkin; Entity NodesOf[All]; }
    }
    Constraint {
      { NameOfCoef t; EntityType NodesOf ; NameOfConstraint InitTemp; }
    }
  }
}

//---------------------------------------------------------------------------------------------

Formulation {

  { Name MagStaDyn_av_js0_3D ; Type FemEquation ;
    Quantity {
      { Name a  ; Type Local ; NameOfSpace HSpace ; }
      { Name v  ; Type Local ; NameOfSpace USpace ; }
    }

    Equation {
      Galerkin { [ nu[] * Dof{d a} , {d a} ] ;
        In Domain ; Jacobian Vol ; Integration II ; }
      Galerkin { DtDof[ sigma[] * Dof{a} , {a} ] ;
        In DomainC ; Jacobian Vol ; Integration II ; }
      Galerkin { [ sigma[] * Dof{d v} , {a} ] ;
        In DomainC ; Jacobian Vol ; Integration II ; }

      Galerkin { [ -js0[], {a} ] ;
        In  DomainS ; Jacobian Vol ; Integration II ; }


      Galerkin{ DtDof[ sigma[] * Dof{a}, {d v} ] ;
	In DomainC ; Jacobian Vol ; Integration II ; }
      Galerkin{ [ sigma[] * Dof{d v} , {d v} ] ;
	In DomainC ; Jacobian Vol ; Integration II ; }

    }
  }
  
  { Name Thermal; Type FemEquation;
    Quantity {
      { Name t; Type Local; NameOfSpace TSpace; }
      { Name a; Type Local; NameOfSpace HSpace; }
      { Name v; Type Local; NameOfSpace USpace; }
    }
    Equation {
      Galerkin { [ K[] * Dof{d t}, {d t} ];
	In DomainC; Integration II; Jacobian Vol; }
      Galerkin { DtDof [ rho[]*Cp[] * Dof{t}, {t} ];
	In DomainC; Integration II; Jacobian Vol; }
      Galerkin { [ -0.5*sigma[]*<a>[SquNorm[Dt[{a}]+{d v}] ], {t} ]; 
	In DomainC; Integration II; Jacobian Vol; }

      Galerkin { [ Ht[]*Dof{t}, {t} ];
	In Skin_ECore; Jacobian Sur ; Integration II ; }
      Galerkin { [-Ht[]*Tamb[], {t} ];
	In Skin_ECore; Jacobian Sur ; Integration II ; }
      Galerkin { [ sigma_sb*Ep[]*({t})^4, {t} ];
	In Skin_ECore; Jacobian Sur ; Integration II ; }
      Galerkin { [ -sigma_sb*Ep[]*(Tamb[])^4, {t} ];
	In Skin_ECore; Jacobian Sur ; Integration II ; }

    }
  }
}

Resolution {

  { Name Analysis ;
    System {
      { Name Thermal ; NameOfFormulation Thermal; }
      { Name Sys ; NameOfFormulation MagStaDyn_av_js0_3D ; Type ComplexValue ; Frequency Freq ; }
    }
    Operation {
      CreateDir[Dir] ;

      InitSolution[Sys] ; Generate[Sys] ; Solve[Sys] ; SaveSolution[Sys] ;
      
      InitSolution[Thermal];
      TimeLoopTheta[ time0, time1, dt[], theta ] {
	IterativeLoop[ NL_NbrMax, NL_Eps, NL_Relax ] {
	  GenerateJac[Thermal];
	  SolveJac[Thermal];
	}
	SaveSolution[Thermal];
      }
      PostOperation[MagDyn] ;
      PostOperation[Thermal] ;

    }
  }
}

//-----------------------------------------------------------------------------------------------

PostProcessing {
  { Name MagStaDyn_av_js0_3D ; NameOfFormulation MagStaDyn_av_js0_3D ; NameOfSystem Sys;
    PostQuantity {
      { Name a ; Value { Term { [ {a} ]          ; In Domain ; Jacobian Vol ; } } }
      { Name h ; Value { Term { [ {d a} ]        ; In Domain ; Jacobian Vol ; } } }
      { Name b ; Value { Term { [ {d a}/nu[] ]   ; In Domain ; Jacobian Vol ; } } }      
      { Name v ; Value { Term { [ {v} ]          ; In DomainC ; Jacobian Vol ; } } }
      { Name e ; Value { Term { [ -(Dt[{a}]+{d v}) ] ; In DomainC ; Jacobian Vol ; } } }
      { Name j ; Value { Term { [ -sigma[]*(Dt[{a}]+{d v}) ] ; In DomainC ; Jacobian Vol ; } } }
      { Name q ; Value { Term { [ 0.5*sigma[]*SquNorm[Dt[{a}]+{d v}] ] ; In DomainC ; Jacobian Vol ; } } }
      { Name js ; Value { Term { [ js0[] ]      ; In DomainS ; Jacobian Vol ; } } }
      
      { Name JouleLosses ;
        Value { Integral {
            [ sigma[]*SquNorm[Dt[{a}]+{d v}] ] ;
            In DomainC ; Jacobian Vol ; Integration II ; }
        }
      }
      { Name MagEnergy ;
        Value { Integral {
            [ 1/2 * nu[{d a}]*{d a} * {d a} ] ;
	    In Domain ; Jacobian Vol ; Integration II ; }
	}
      }

      { Name Flux ; Value {
          Integral { [ js0[]*{a} ] ;
            In Inds  ; Jacobian Vol ; Integration II ; }
        }
      }

      { Name Upos ;
        Value { Integral { Type Global ;
            [ -sigma[] * (Dt[{a}] + {d v}) * BF{d v} ] ;
            In DomainC ; Jacobian Vol ; Integration II ;
          }
        }
      }

    }
  }
  { Name Thermal ; NameOfFormulation Thermal ; NameOfSystem Thermal;
    Quantity {
      { Name t; Value{ Local{ [ {t} ] ;
	    In DomainC ; Jacobian Vol; } } }
      { Name p; Value{ Local{ [ -K[]*{ Grad t } ] ;
	    In DomainC ; Jacobian Vol; } } }
    }
  }
}

//-----------------------------------------------------------------------------------------------
PostOperation {
  { Name MagDyn ; NameOfPostProcessing MagStaDyn_av_js0_3D ;
    Operation {
      Print[ js, OnElementsOf DomainS, File StrCat[Dir, "js", ExtGmsh], LastTimeStepOnly ] ;
      Print[ a, OnElementsOf Domain, File StrCat[Dir, "a", ExtGmsh], LastTimeStepOnly ] ;
      Print[ v, OnElementsOf Domain, File StrCat[Dir,"v",ExtGmsh], LastTimeStepOnly ] ;
      Print[ b, OnElementsOf Domain, File StrCat[Dir,"b",ExtGmsh], LastTimeStepOnly ] ;
      Print[ h, OnElementsOf Domain, File StrCat[Dir,"h",ExtGmsh], LastTimeStepOnly ] ;
      Print[ q, OnElementsOf DomainC, File StrCat[Dir,"q",ExtGmsh], LastTimeStepOnly ] ;
      Print[ j, OnElementsOf DomainC, File StrCat[Dir,"j",ExtGmsh], LastTimeStepOnly ] ;
    }
  }
  { Name Thermal ; NameOfPostProcessing Thermal ; 
    Operation {
      Print[ t, OnElementsOf DomainC, File StrCat[Dir, "tThe", ExtGmsh], LastTimeStepOnly ] ;
      Print[ p, OnElementsOf DomainC, File StrCat[Dir, "pThe", ExtGmsh], LastTimeStepOnly ] ;
      //      Print[ qq, OnElementsOf DomainC, File StrCat[Dir, "qThe", ExtGmsh], LastTimeStepOnly ] ;
    }
  }
}

DefineConstant[
  R_ = {"Analysis", Name "GetDP/1ResolutionChoices", Visible 1},
  C_ = {"-solve -v2", Name "GetDP/9ComputeCommand", Visible 1},
  P_ = {"Thermal", Name "GetDP/2PostOperationChoices", Visible 1}
];
